﻿using DataAccess;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using ViewModels.Provider;

/// <summary>
/// Summary description for EncounterDataManager
/// </summary>
namespace Managers.Encounter
{
    /// <summary>
    /// The EncounterDataManager class handles data access for the Encounter records in the database.
    /// </summary>
    public class EncounterDataManager
    {
        #region Member Variables

        private const string CONNECTION_NAME = "RegistryConnectionString";
        private const string GET_FACILITY_NAME_AND_STA3N_BY_ID = "MEVIR.usp_Get_Institution_Name_and_Sta3n_By_ID";
        private const string GET_ENCOUNTERS_BY_STATUS_ID = "MEVIR.usp_GetEncountersByStatusId";

        #endregion

        /// <summary>
        /// Default Constructor
        /// </summary>
        public EncounterDataManager()
        {
        }

        /// <summary>
        /// GetEncounterById: Returns the ER_ENCOUNTER_MOD record with the passed id (null if not found)
        /// </summary>
        /// <param name="encounterId"></param>
        /// <returns></returns>
        public static ER_ENCOUNTER_MOD GetEncounterById(int encounterId)
        {
            ER_ENCOUNTER_MOD encounter = null;

            using (var ctx = new MEVIREntities())
            {
                encounter = (from e in ctx.ER_ENCOUNTER_MOD
                            where e.ER_ENCOUNTER_ID == encounterId
                            select e).FirstOrDefault();

                return encounter;
            }
        }

        /// <summary>
        /// GetEncountersByCriteria: returns a list of encounters with the specified status
        /// </summary>
        /// <param name="statusId"></param>
        /// <returns></returns>
        public static DataTable GetEncountersByStatusId(int statusId)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_ENCOUNTERS_BY_STATUS_ID, new object[] { statusId });
        }

        /// <summary>
        /// GetProvidersForVisitIdentifier: returns a list of EncounterProvider for an encounter unique visit identifier
        /// </summary>
        /// <param name="encounterId"></param>
        /// <returns></returns>
        public static List<EncounterProvider> GetProvidersForVisitIdentifier(string visitIdentifier)
        {
            using (var ctx = new MEVIREntities())
            {
                var providers = from s in ctx.ENCOUNTER_STAFF
                                join p in ctx.PROVIDERS on s.StaffSID equals p.StaffSID into ps
                                from p in ps.DefaultIfEmpty()
                                where s.VisitIdentifier == visitIdentifier
                                select new EncounterProvider() 
                                { 
                                    Id = s.ID, 
                                    BusinessPhone = p.OfficePhone, 
                                    City = p.City, 
                                    Fax = p.FaxNumber, 
                                    InstitutionName = p.InstitutionName, 
                                    InstitutionShortName = p.InstitutionShortName, 
                                    Sta3n = p.Sta3n, 
                                    StaffName = p.StaffName, 
                                    StaffSID = p.StaffSID, 
                                    StateAbbrev = p.StateAbbrev, 
                                    StreetAddress1 = p.StreetAddress1, 
                                    StreetAddress2 = p.StreetAddress2, 
                                    VisitIdentifier = s.VisitIdentifier, 
                                    Zip = p.Zip
                                };

                if (providers != null)
                {
                    return providers.ToList();
                }

                //var q =
                //        from c in categories
                //        join p in products on c equals p.Category into ps
                //        from p in ps.DefaultIfEmpty()
                //        select new { Category = c, ProductName = p == null ? "(No products)" : p.ProductName };
            }

            return null;
        }

        /// <summary>
        /// Returns all providers for the system.
        /// </summary>
        /// <returns></returns>
        public static List<ProviderStaffListDisplay> GetAllProviders(string sta3n)
        {
            using (var ctx = new MEVIREntities()) 
            {
                if (!String.IsNullOrEmpty(sta3n))
                {
                    int station = Int16.Parse(sta3n);

                    var list = from p in ctx.PROVIDERS
                               where p.Sta3n == station
                               orderby p.StaffName
                               select new ProviderStaffListDisplay() { StaffSID = p.StaffSID, DisplayStaffName = p.StaffName };

                    if (list != null)
                    {
                        return list.ToList();
                    }
                }
                else
                {
                    var list = from p in ctx.PROVIDERS
                               orderby p.StaffName
                               select new ProviderStaffListDisplay() { StaffSID = p.StaffSID, DisplayStaffName = p.StaffName };
                    
                    if (list != null)
                    {
                        return list.ToList();
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// Inserts a new record for the encounter into ENCOUNTER_STAFF
        /// </summary>
        /// <param name="staff"></param>
        public static void InsertEncounterProvider(ENCOUNTER_STAFF staff)
        {
            using (var ctx = new MEVIREntities())
            {
                ctx.ENCOUNTER_STAFF.Add(new ENCOUNTER_STAFF() { StaffName = staff.StaffName, StaffSID = staff.StaffSID, VisitIdentifier = staff.VisitIdentifier });
                ctx.SaveChanges();
            }
        }

        /// <summary>
        /// Deletes the ENCOUNTER_STAFF with the corresponding ID
        /// </summary>
        /// <param name="id"></param>
        public static void DeleteEncounterProvider(int id)
        {
            using (var ctx = new MEVIREntities())
            {
                ctx.ENCOUNTER_STAFF.Remove(ctx.ENCOUNTER_STAFF.FirstOrDefault(S => S.ID == id));
                ctx.SaveChanges();
            }
        }

        /// <summary>
        /// GetFacilityNameById: Returns a facility name by Id
        /// </summary>
        /// <param name="facilityId"></param>
        /// <returns></returns>
        public static DataRow GetFacilityNameAndSta3nById(int? facilityId)
        {
            if (facilityId == null)
            {
                return null;
            }

            DataTable dt = SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_FACILITY_NAME_AND_STA3N_BY_ID, new object[] { facilityId });
            if (dt != null)
            {
                if (dt.Rows.Count > 0)
                {
                    return dt.Rows[0];
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// InsertEncounter: used to "create" manually entered encounter record.
        /// </summary>
        /// <param name="referralId"></param>
        /// <param name="institutionId"></param>
        /// <param name="visitDate"></param>
        /// <param name="encounterType"></param>
        public static void InsertEncounter(int currentUserId, string currentUserName, int referralId, int patientSid, int institutionId, DateTime visitDate, string encounterType)
        {
            using (var ctx = new MEVIREntities())
            {
                DataRow dr = EncounterDataManager.GetFacilityNameAndSta3nById(institutionId);
                string sta6n = (dr["STATIONNUMBER"] != null) ? dr["STATIONNUMBER"].ToString() : "";
                ctx.ER_ENCOUNTER_MOD.Add(new ER_ENCOUNTER_MOD() 
                { 
                    REFERRAL_ID = referralId,
                    PatientSID = patientSid,
                    CREATED = DateTime.Now, 
                    CREATEDBY = currentUserName, 
                    EncounterStatusID = 1, 
                    EncounterTypeInOut = encounterType, 
                    InstitutionID = institutionId, 
                    STA6A = sta6n, 
                    UniqueVHAEncounterIDPrimaryKey = Guid.NewGuid().ToString(), 
                    UPDATED = DateTime.Now, 
                    UPDATEDBY = currentUserName, 
                    VISIT_DATE = visitDate, 
                    AssignedTo = currentUserId, 
                    AssignedDate = DateTime.Now
                });

                ctx.SaveChanges();
            }
        }

        /// <summary>
        /// SaveEncounter: This method will take a passed ER_ENCOUNTER_MOD record and save it to the database (updates).
        /// </summary>
        /// <param name="currentUserName"></param>
        /// <param name="passedEncounterRecord"></param>
        public static void SaveEncounter(int currentUserId, string currentUserName, ER_ENCOUNTER_MOD passedEncounterRecord)
        {
            using (var ctx = new MEVIREntities())
            {
                ER_ENCOUNTER_MOD originalEncounter = ctx.ER_ENCOUNTER_MOD.FirstOrDefault(E => E.ER_ENCOUNTER_ID == passedEncounterRecord.ER_ENCOUNTER_ID);

                if (originalEncounter != null)
                {
                    // map columns and update -- general data
                    originalEncounter.UPDATED = DateTime.Now;
                    originalEncounter.UPDATEDBY = currentUserName;
                    originalEncounter.EncounterTypeInOut = passedEncounterRecord.EncounterTypeInOut;
                    originalEncounter.InstitutionID = passedEncounterRecord.InstitutionID;
                    originalEncounter.STA6A = passedEncounterRecord.STA6A;
                    originalEncounter.VISIT_DATE = passedEncounterRecord.VISIT_DATE;

                    // complaint tab
                    originalEncounter.CHIEF_COMPLAINT_TEXT = passedEncounterRecord.CHIEF_COMPLAINT_TEXT;
                    originalEncounter.TBI_IND = passedEncounterRecord.TBI_IND;
                    originalEncounter.TBI_READ_PRBLM_IND = passedEncounterRecord.TBI_READ_PRBLM_IND;
                    originalEncounter.TBI_DIPLOPIA_IND = passedEncounterRecord.TBI_DIPLOPIA_IND;
                    originalEncounter.TBI_DAZZLING_IND = passedEncounterRecord.TBI_DAZZLING_IND;
                    originalEncounter.TBI_PHOTOPHOBIA_IND = passedEncounterRecord.TBI_PHOTOPHOBIA_IND;
                    originalEncounter.TBI_EYE_STRAIN_IND = passedEncounterRecord.TBI_EYE_STRAIN_IND;
                    originalEncounter.TBI_BLURRED_VISION_IND = passedEncounterRecord.TBI_BLURRED_VISION_IND;
                    originalEncounter.OTHER_NEUROLOGICAL_IND = passedEncounterRecord.OTHER_NEUROLOGICAL_IND;
                    originalEncounter.NEUROLOGICAL_COMMENT_TEXT = passedEncounterRecord.NEUROLOGICAL_COMMENT_TEXT;

                    // Diagnosis Tab
                    originalEncounter.BLIND_REHAB_INDICATOR = passedEncounterRecord.BLIND_REHAB_INDICATOR;

                    // ALWAYS Set Status to INPROC IF - original status is NEW
                    if (originalEncounter.EncounterStatusID == EncounterStatusType.NEW)
                    {
                        originalEncounter.EncounterStatusID = EncounterStatusType.IN_PROC;
                    }

                    if (originalEncounter.AssignedTo == null || originalEncounter.AssignedTo == currentUserId)
                    {
                        originalEncounter.AssignedTo = currentUserId;
                    }


                    if (originalEncounter.AssignedDate == null)
                    {
                        originalEncounter.AssignedDate = DateTime.Now;
                    }

                    ctx.SaveChanges();
                }
            }
        }

        /// <summary>
        /// Updates an Encounter's status
        /// </summary>
        /// <param name="currentUserName"></param>
        /// <param name="encounterId"></param>
        /// <param name="statusId"></param>
        public static void UpdateEncounterStatus(string currentUserName, int encounterId, int statusId)
        {
            using (var ctx = new MEVIREntities())
            {
                ER_ENCOUNTER_MOD originalEncounter = ctx.ER_ENCOUNTER_MOD.FirstOrDefault(E => E.ER_ENCOUNTER_ID == encounterId);

                if (originalEncounter != null)
                {
                    originalEncounter.UPDATED = DateTime.Now;
                    originalEncounter.UPDATEDBY = currentUserName;
                    originalEncounter.EncounterStatusID = statusId;
                }

                ctx.SaveChanges();
            }
        }

        #region ENCOUNTER DETAILS

        /// <summary>
        /// GetEncounterDiagnosisEntries: returns a list of ENCOUNTER_DETAIL object for the given Encounter that are ICD codes
        /// </summary>
        /// <param name="visitIdentifier"></param>
        /// <returns></returns>
        public static List<ENCOUNTER_DETAILS> GetEncounterDetailsByEncounterAndType(string visitIdentifier, string detailType)
        {
            using (var ctx = new MEVIREntities())
            {
                var details = from d in ctx.ENCOUNTER_DETAILS
                              where d.VisitIdentifier == visitIdentifier && d.Type == detailType
                              select d;

                if (details != null)
                {
                    return details.ToList();
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the associated Encounter Detail record.
        /// </summary>
        /// <param name="id"></param>
        public static ENCOUNTER_DETAILS GetEncounterDetailById(int id)
        {
            using (var ctx = new MEVIREntities())
            {
                return ctx.ENCOUNTER_DETAILS.FirstOrDefault(D => D.ID == id);
            }
        }

        /// <summary>
        /// Inserts a new ENCOUNTER_DETAIL record into the system for the given encounter.
        /// </summary>
        /// <param name="visitIdentifier"></param>
        /// <param name="detailType"></param>
        /// <param name="detailValue"></param>
        /// <param name="detailName"></param>
        /// <param name="description"></param>
        public static void InsertEncounterDetail(string visitIdentifier, string detailType, string detailValue, string detailName, string description)
        {
            using (var ctx = new MEVIREntities())
            {
                ctx.ENCOUNTER_DETAILS.Add(new ENCOUNTER_DETAILS() { Type = detailType, Name = detailName, Value = detailValue, Description = description, VisitIdentifier = visitIdentifier });
                ctx.SaveChanges();
            }
        }

        /// <summary>
        /// Updates an Encounter Detail record with the new information.
        /// </summary>
        /// <param name="passedDetails"></param>
        public static void UpdateEncounterDetail(ENCOUNTER_DETAILS passedDetails)
        {
            using (var ctx = new MEVIREntities())
            {
                ENCOUNTER_DETAILS originalDetails = ctx.ENCOUNTER_DETAILS.FirstOrDefault(E => E.ID == passedDetails.ID);

                if (originalDetails != null)
                {
                    originalDetails.Description = passedDetails.Description;
                    originalDetails.Name = passedDetails.Name;
                    originalDetails.Type = passedDetails.Type;
                    originalDetails.Value = passedDetails.Value;
                    originalDetails.VisitIdentifier = passedDetails.VisitIdentifier;

                    ctx.SaveChanges();
                }
            }
        }

        /// <summary>
        /// Deletes the associated Encounter Detail record.
        /// </summary>
        /// <param name="id"></param>
        public static void DeleteEncounterDetailById(int id)
        {
            using (var ctx = new MEVIREntities())
            {
                ctx.ENCOUNTER_DETAILS.Remove(ctx.ENCOUNTER_DETAILS.FirstOrDefault(S => S.ID == id));
                ctx.SaveChanges();
            }
        }

        #endregion

        #region ENCOUNTER REVIEW COMMENTS

        /// <summary>
        /// Returns all comments for a given Encounter
        /// </summary>
        /// <param name="encounterId"></param>
        /// <returns></returns>
        public static List<ER_ENCOUNTER_REVIEWCOMMENT> GetAllReviewCommentsByEncounterId(int encounterId)
        {
            using (var ctx = new MEVIREntities())
            {
                var list = from c in ctx.ER_ENCOUNTER_REVIEWCOMMENT
                           where c.ER_ENCOUNTER_MOD_ID == encounterId
                           select c;

                if (list != null)
                {
                    return list.ToList();
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// GetReviewCommentByEncounterIdAndQuestionCode: Returns the review comment for the passed data - if none, returns null
        /// </summary>
        /// <param name="encounterId"></param>
        /// <param name="questionCode"></param>
        /// <returns></returns>
        public static ER_ENCOUNTER_REVIEWCOMMENT GetReviewCommentByEncounterIdAndQuestionCode(int encounterId, string questionCode) {
            using (var ctx = new MEVIREntities())
            {
                return ctx.ER_ENCOUNTER_REVIEWCOMMENT.FirstOrDefault(C => C.ER_ENCOUNTER_MOD_ID == encounterId && C.QUESTION_CODE == questionCode);
            }
        }

        public static ER_ENCOUNTER_REVIEWCOMMENT GetReviewCommentByEncounterIdAndQuestionCodeAndLaterality(int encounterId, string questionCode, string lateralityCode) {
            using (var ctx = new MEVIREntities())
            {
                return ctx.ER_ENCOUNTER_REVIEWCOMMENT.FirstOrDefault(
                    C => C.ER_ENCOUNTER_MOD_ID == encounterId && 
                        C.QUESTION_CODE == questionCode &&
                        C.LATERALITY_CODE == lateralityCode);
            }
        }

        /// <summary>
        /// Save the Review Comment - Insert / Update
        /// </summary>
        /// <param name="reviewComment"></param>
        /// <param name="currentUserName"></param>
        public static void SaveReviewComment(ER_ENCOUNTER_REVIEWCOMMENT reviewComment, string currentUserName)
        {
            // ID, LATERALITY_CODE, ER_ENCOUNTER_MOD_ID, DATE_ENTERED, RESOLVED_FLAG, RESOLVED_BY, DATE_RESOLVED, APPROVED_FLAG, APPROVED_BY, DATE_APPROVED, QUESTION_CODE, CREATED, CREATEDBY, UPDATED, UPDATEDBY, COMMENT_TEXT
            using (var ctx = new MEVIREntities())
            {
                if (reviewComment.ID == 0)
                {
                    // insert
                    ctx.ER_ENCOUNTER_REVIEWCOMMENT.Add(new ER_ENCOUNTER_REVIEWCOMMENT() 
                    { 
                        COMMENT_TEXT = reviewComment.COMMENT_TEXT, 
                        CREATED = DateTime.Now, 
                        CREATEDBY = currentUserName, 
                        DATE_ENTERED = DateTime.Now, 
                        ER_ENCOUNTER_MOD_ID = reviewComment.ER_ENCOUNTER_MOD_ID, 
                        LATERALITY_CODE = reviewComment.LATERALITY_CODE, 
                        QUESTION_CODE = reviewComment.QUESTION_CODE,  
                        ANCHOR_ID = reviewComment.ANCHOR_ID, 
                        LOCATION_FRIENDLY_NAME = reviewComment.LOCATION_FRIENDLY_NAME, 
                        UPDATED = DateTime.Now, 
                        UPDATEDBY = currentUserName
                    });
                }
                else
                {
                    // update
                    ER_ENCOUNTER_REVIEWCOMMENT originalComment = ctx.ER_ENCOUNTER_REVIEWCOMMENT.FirstOrDefault(E => E.ID == reviewComment.ID);

                    if (originalComment != null)
                    {
                        originalComment.APPROVED_BY = reviewComment.APPROVED_BY;
                        originalComment.APPROVED_FLAG = reviewComment.APPROVED_FLAG;
                        originalComment.COMMENT_TEXT = reviewComment.COMMENT_TEXT;
                        originalComment.DATE_APPROVED = reviewComment.DATE_APPROVED;
                        originalComment.DATE_RESOLVED = reviewComment.DATE_RESOLVED;
                        originalComment.LATERALITY_CODE = reviewComment.LATERALITY_CODE;
                        originalComment.RESOLVED_BY = reviewComment.RESOLVED_BY;
                        originalComment.RESOLVED_FLAG = reviewComment.RESOLVED_FLAG;
                        originalComment.UPDATED = DateTime.Now;
                        originalComment.UPDATEDBY = currentUserName;
                        originalComment.ANCHOR_ID = reviewComment.ANCHOR_ID;
                        originalComment.LOCATION_FRIENDLY_NAME = reviewComment.LOCATION_FRIENDLY_NAME;
                    }
                }

                ctx.SaveChanges();
            }
        }

        /// <summary>
        /// Takes a pages control collection and searches recursively for all controls of passed type.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="controlCollection"></param>
        /// <param name="resultCollection"></param>
        public static void GetControlList<T>(ControlCollection controlCollection, List<T> resultCollection) where T : Control
        {
            foreach (Control control in controlCollection)
            {
                //if (control.GetType() == typeof(T))
                if (control is T) // This is cleaner
                    resultCollection.Add((T)control);

                if (control.HasControls())
                    GetControlList(control.Controls, resultCollection);
            }
        }

        #endregion
    }
}