﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels;
using ViewModels.Exam;
using VeteransAffairs.Registries.BusinessManager;

namespace Managers.Exam
{
    public abstract class BaseEventViewManager<TResult> : IEventViewManager
        where TResult : BaseEventModel
    {
        protected DataAccessManager DataAccessManager { get; private set; }

        protected BaseEventViewManager()
        {
            this.DataAccessManager = new DataAccessManager();
        }

        #region Abstract Methods

        public abstract TResult GetDefaultViewModel();
        public abstract TResult GetLoadedViewModel(int patientId, int encounterId, string lateralityCode);
        public abstract void Save(TResult result, int patientId, int encounterId);

        #endregion

        #region IEventViewManager Implementations

        BaseEventModel IEventViewManager.GetDefaultViewModel()
        {
            return GetDefaultViewModel();
        }

        BaseEventModel IEventViewManager.GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            return GetLoadedViewModel(patientId, encounterId, lateralityCode);
        }

        void IEventViewManager.Save(BaseEventModel eventModel, int patientId, int encounterId)
        {
            TResult result = eventModel as TResult;
            if (result != null)
            {
                Save(result, patientId, encounterId);
            }
        }

        public void Delete(int patientId, int encounterId, string lateralityCode, string categoryCode)
        {
            // Find Responses to remove
            List<int> responsesToBeRemoved =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                .Select(r => r.RESPONSE_ID)
                .ToList();

            DataAccessManager.RemoveResponses(responsesToBeRemoved);
        }

        #endregion

        #region Database Entity Parsers

        private QUESTION FindQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            return questionList.FirstOrDefault(q => q.QUESTION_CODE == questionCode);
        }

        private RESPONSE FindResponseByQuestionCode(IEnumerable<RESPONSE> responseList, string questionCode)
        {
            return responseList.FirstOrDefault(r => r.QUESTION_CODE == questionCode);
        }

        private IEnumerable<RESPONSE> FindResponsesByQuestionCode(IEnumerable<RESPONSE> responseList,
            string questionCode)
        {
            return responseList.Where(r => r.QUESTION_CODE == questionCode);
        }

        #endregion

        #region To View Model Mappers

        protected List<Laterality> MapToViewModelLateralities(IEnumerable<LATERALITY> lateralities)
        {
            return lateralities.Select(MapToViewModelLaterality).ToList();
        }

        protected Laterality MapToViewModelLaterality(LATERALITY laterality)
        {
            return new Laterality()
            {
                Code = laterality.LATERALITY_CODE,
                DisplayText = laterality.LATERALITY_NAME,
                SortOrder = laterality.LATERALITY_ID
            };
        }

        protected Laterality MapToViewModelLaterality(IEnumerable<LATERALITY> lateralityList, string specifiedCode)
        {
            var laterality = lateralityList.First(l => l.LATERALITY_CODE == specifiedCode);
            return MapToViewModelLaterality(laterality);
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            var singleSelectQuestion = new SingleSelectQuestion();
            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, singleSelectQuestion);

            singleSelectQuestion.AllChoices.AddRange(MapToChoiceViewModels(question.QUESTION_CHOICES));

            return singleSelectQuestion;
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode, string selectedChoiceCode)
        {
            var singleSelectQuestion = MapToSingleSelectQuestion(questionList, questionCode);

            singleSelectQuestion.SelectedChoice =
                singleSelectQuestion.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

            return singleSelectQuestion;
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            SingleSelectQuestion singleSelectQuestion = MapToSingleSelectQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                singleSelectQuestion.SelectedChoice = MapToChoiceViewModel(response.QUESTION_CHOICE);
            }

            return singleSelectQuestion;
        }

        protected TextQuestion MapToTextQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            TextQuestion textQuestion = new TextQuestion();

            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, textQuestion);

            return textQuestion;
        }

        protected TextQuestion MapToTextQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            TextQuestion textQuestion = MapToTextQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                textQuestion.ResponseText = response.RESPONSE_TEXT;
            }

            return textQuestion;
        }
        
        protected Dictionary<string, TextQuestion> MapToTextQuestionsMultiSelectSub(
            IEnumerable<QUESTION> questionList, 
            MultiSelectQuestion multiSelectQuestion, 
            string questionSuffix)
        {
            Dictionary<string, TextQuestion> textQuestions = new Dictionary<string, TextQuestion>();

            foreach (Choice choice in multiSelectQuestion.AllChoices)
            {
                string questionCode = choice.Code + questionSuffix;
                TextQuestion textQuestion = MapToTextQuestion(questionList, questionCode);

                textQuestions.Add(choice.Code, textQuestion);
            }

            return textQuestions;
        }
        
        protected Dictionary<string, TextQuestion> MapToTextQuestionsMultiSelectSub(
            IEnumerable<QUESTION> questionList,
            IEnumerable<RESPONSE> responseList,
            MultiSelectQuestion multiSelectQuestion, 
            string questionSuffix)
        {
            Dictionary<string, TextQuestion> textQuestions = new Dictionary<string, TextQuestion>();

            foreach (Choice choice in multiSelectQuestion.AllChoices)
            {
                string questionCode = choice.Code + questionSuffix;
                TextQuestion textQuestion = MapToTextQuestion(questionList, responseList, questionCode);

                textQuestions.Add(choice.Code, textQuestion);
            }

            return textQuestions;
        }

        protected YesNoQuestion MapToYesNoQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            YesNoQuestion yesNoQuestion = new YesNoQuestion();

            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, yesNoQuestion);

            return yesNoQuestion;
        }

        protected YesNoQuestion MapToYesNoQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            YesNoQuestion yesNoQuestion = MapToYesNoQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                yesNoQuestion.ResponseText = response.RESPONSE_TEXT;
            }

            return yesNoQuestion;
        }

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            var multiSelectQuestion = new MultiSelectQuestion();
            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, multiSelectQuestion);

            multiSelectQuestion.AllChoices.AddRange(MapToChoiceViewModels(question.QUESTION_CHOICES));

            return multiSelectQuestion;
        }

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            var multiSelectQuestion = MapToMultiSelectQuestion(questionList, questionCode);

            IEnumerable<RESPONSE> responses = FindResponsesByQuestionCode(responseList, questionCode).ToList();

            if (responses.Any())
            {
                multiSelectQuestion.SelectedChoices.AddRange(MapToChoiceViewModels(responses));
            }

            return multiSelectQuestion;
        }
        
        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<QUESTION_CHOICE> questionChoices)
        {
            return questionChoices.Select(MapToChoiceViewModel);
        }

        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<RESPONSE> responses)
        {
            return responses.Select(r => MapToChoiceViewModel(r.QUESTION_CHOICE, r.COMMENT));
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice)
        {
            return MapToChoiceViewModel(questionChoice, string.Empty);
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice, string commentText)
        {
            var choice = new Choice
            {
                Code = questionChoice.QUESTION_CHOICE_CODE,
                DisplayText = questionChoice.DISPLAY_TEXT,
                SortOrder = questionChoice.SORT_ORDER,
                RequiresComment = questionChoice.COMMENT_REQUIRED,
                Comment = commentText
            };

            return choice;
        }

        private void InitializeQuestion(QUESTION dbQuestion, BaseQuestion baseQuestion)
        {
            baseQuestion.Code = dbQuestion.QUESTION_CODE;
            baseQuestion.DisplayText = dbQuestion.QUESTION_TEXT;
        }

        #endregion

        #region To Database Entity Mappers

        protected RESPONSE MapSingleSelectQuestionToResponse(SingleSelectQuestion question, int patientId,
            int encounterId, string lateralityCode)
        {
            RESPONSE response = null;
            if (question.IsAnswered)
            {
                response = MapBaseQuestionToResponse(question, patientId, encounterId, lateralityCode);
                response.QUESTION_CHOICE_CODE = question.SelectedChoice.Code;
                response.COMMENT = question.SelectedChoice.RequiresComment ? question.SelectedChoice.Comment : null;
            }

            return response;
        }

        protected RESPONSE MapTextQuestionToResponse(TextQuestion question, int patientId, int encounterId, string lateralityCode)
        {
            RESPONSE response = null;
            if (question.IsAnswered)
            {
                response = MapBaseQuestionToResponse(question, patientId, encounterId, lateralityCode);
                response.RESPONSE_TEXT = question.ResponseText;
            }

            return response;
        }

        protected RESPONSE MapYesNoQuestionToResponse(YesNoQuestion question, int patientId, int encounterId, string lateralityCode)
        {
            return MapTextQuestionToResponse(question, patientId, encounterId, lateralityCode);
        }

        protected IEnumerable<RESPONSE> MapMultiSelectQuestionToResponse(MultiSelectQuestion question, int patientId, int encounterId, string lateralityCode)
        {
            List<RESPONSE> responses = new List<RESPONSE>();
            if (question.IsAnswered)
            {
                foreach (Choice choice in question.SelectedChoices)
                {
                    RESPONSE response = MapBaseQuestionToResponse(question, patientId, encounterId, lateralityCode);
                    response.QUESTION_CHOICE_CODE = choice.Code;
                    response.COMMENT = choice.RequiresComment ? choice.Comment : null;

                    responses.Add(response);
                }
            }

            return responses;
        }
        
        protected IEnumerable<RESPONSE> MapTextQuestionMultiSelectSubsToResponses(
            Dictionary<string, TextQuestion> textQuestions, 
            MultiSelectQuestion multiSelectQuestion, 
            int patientId, 
            int encounterId, 
            string lateralityCode)
        {
            return multiSelectQuestion.SelectedChoices.Select(
                c =>
                {
                    var textQuestion = textQuestions[c.Code];
                    return MapTextQuestionToResponse(textQuestion, patientId, encounterId, lateralityCode);
                }).ToList();
        }

        private RESPONSE MapBaseQuestionToResponse(BaseQuestion question, int patientId, int encounterId,
            string lateralityCode)
        {
            UserAccountManager uam = ((UserAccountManager)System.Web.HttpContext.Current.User);

            RESPONSE response = new RESPONSE()
            {
                ENCOUNTER_ID = encounterId,
                REFERRAL_ID = patientId,
                QUESTION_CODE = question.Code,
                LATERALITY_CODE = lateralityCode,
                CREATED = DateTime.Now,
                CREATED_BY = uam.UserName,
                UPDATED = DateTime.Now,
                UPDATED_BY = uam.UserName
            };

            return response;
        }

        #endregion
    }
}