﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class MotilityEventViewManager : BaseEventViewManager<MotilityEventModel>
    {
        public override MotilityEventModel GetDefaultViewModel()
        {
            var viewModel = new MotilityEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.TestPerformed,
                MotilityEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.TestResult,
                MotilityEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.EventComment);

            viewModel.Pursuit = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Pursuit,
                MotilityEventModel.ChoiceCodes.Pursuit.NotDocumented);

            viewModel.Saccades = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Saccades,
                MotilityEventModel.ChoiceCodes.Saccades.NotDocumented);

            viewModel.Nystagmus = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Nystagmus,
                MotilityEventModel.ChoiceCodes.Nystagmus.NotDocumented);

            viewModel.NystagmusOptions = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.NystagmusOptions);
            viewModel.Motility = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.Motility);
            viewModel.CranialNerve = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.CranialNerve);

            return viewModel;
        }

        public override MotilityEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new MotilityEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.TestResult);
                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.EventComment);
                viewModel.Pursuit = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Pursuit);
                viewModel.Saccades = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Saccades);
                viewModel.Nystagmus = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Nystagmus);
                viewModel.NystagmusOptions = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.NystagmusOptions);
                viewModel.Motility = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Motility);
                viewModel.CranialNerve = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.CranialNerve);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(MotilityEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    MotilityEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Pusuit
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Pursuit, patientId, encounterId,
                        lateralityCode));

                    // Saccades
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Saccades, patientId, encounterId,
                        lateralityCode));

                    // Nystagmus
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Nystagmus, patientId, encounterId,
                        lateralityCode));

                    if (result.Nystagmus.IsAnswered &&
                        result.Nystagmus.SelectedChoice.Code == MotilityEventModel.ChoiceCodes.Nystagmus.Present)
                    {
                        // Nystagmus Options
                        responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.NystagmusOptions, patientId,
                            encounterId, lateralityCode));
                    }

                    // Motility
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.Motility, patientId, encounterId,
                        lateralityCode));

                    // Cranial Nerve
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.CranialNerve, patientId,
                        encounterId, lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}