﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Prosthetic Devices Event View Manager
    /// </summary>
    public class ProstheticDevicesEventViewManager : BaseEventViewManager<ProstheticDevicesEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The orbit view model</returns>
        public override ProstheticDevicesEventModel GetDefaultViewModel()
        {
            var viewModel = new ProstheticDevicesEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.Options = MapToMultiSelectQuestion(categoryQuestionList, ProstheticDevicesEventModel.QuestionCodes.Options);
            viewModel.ImplantTypes = MapToMultiSelectQuestion(categoryQuestionList, ProstheticDevicesEventModel.QuestionCodes.ImplantTypes);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The orbit view model</returns>
        public override ProstheticDevicesEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new ProstheticDevicesEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.Options = MapToMultiSelectQuestion(categoryQuestionList, responseList, ProstheticDevicesEventModel.QuestionCodes.Options);
                viewModel.ImplantTypes = MapToMultiSelectQuestion(categoryQuestionList, responseList, ProstheticDevicesEventModel.QuestionCodes.ImplantTypes);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(ProstheticDevicesEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();
                
                // Options
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.Options, patientId, encounterId, lateralityCode));
                
                // Implant Types
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.ImplantTypes, patientId, encounterId, lateralityCode));

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}