﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Pupil Event View Manager
    /// </summary>
    public class PupilEventViewManager : BaseEventViewManager<PupilEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The Pupil view model</returns>
        public override PupilEventModel GetDefaultViewModel()
        {
            var viewModel = new PupilEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                PupilEventModel.QuestionCodes.TestPerformed,
                PupilEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                PupilEventModel.QuestionCodes.TestResult,
                PupilEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.EventComment);
            viewModel.Size = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Size);
            viewModel.AlterationInShape = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.AlterationInShape);
            viewModel.PerrlOrPerrla = MapToSingleSelectQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.PerrlOrPerrla);
            viewModel.Reactive = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Reactive);
            viewModel.Adie = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Adie);
            viewModel.ArgyllRobertson = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.ArgyllRobertson);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The Pupil view model</returns>
        public override PupilEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new PupilEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.TestResult);
                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.EventComment);

                viewModel.Size = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Size);
                viewModel.AlterationInShape = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.AlterationInShape);
                viewModel.PerrlOrPerrla = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.PerrlOrPerrla);
                viewModel.Reactive = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Reactive);
                viewModel.Adie = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Adie);
                viewModel.ArgyllRobertson = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.ArgyllRobertson);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(PupilEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    PupilEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId, lateralityCode));

                    // Size
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.Size, patientId, encounterId, lateralityCode));

                    // Alteration In Shape
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.AlterationInShape, patientId, encounterId, lateralityCode));

                    // PERRL or PERRLA
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.PerrlOrPerrla, patientId, encounterId, lateralityCode));

                    if (result.PerrlOrPerrla.IsAnswered)
                    {
                        // Reactive
                        responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.Reactive, patientId, encounterId, lateralityCode));
                    }

                    // Adie's
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.Adie, patientId, encounterId, lateralityCode));

                    // Argyll-Robertson
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.ArgyllRobertson, patientId, encounterId, lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}