﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Pupil Event View Manager
    /// </summary>
    public class RelativeAfferentPupilDefectEventViewManager : BaseEventViewManager<RelativeAfferentPupilDefectEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The Pupil view model</returns>
        public override RelativeAfferentPupilDefectEventModel GetDefaultViewModel()
        {
            var viewModel = new RelativeAfferentPupilDefectEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                RelativeAfferentPupilDefectEventModel.QuestionCodes.TestPerformed,
                RelativeAfferentPupilDefectEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                RelativeAfferentPupilDefectEventModel.QuestionCodes.TestResult,
                RelativeAfferentPupilDefectEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToYesNoQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.VisitData);
            viewModel.ADP = MapToSingleSelectQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADP);
            viewModel.ADPLaterality = MapToTextQuestion(categoryQuestionList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADPLaterality);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The Pupil view model</returns>
        public override RelativeAfferentPupilDefectEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new RelativeAfferentPupilDefectEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = 
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestPerformed);

                viewModel.TestResult = 
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.TestResult);

                viewModel.EventComment = 
                    MapToTextQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.EventComment);

                viewModel.VisitData = 
                    MapToYesNoQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.VisitData);

                viewModel.ADP =
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADP);

                viewModel.ADPLaterality =
                    MapToTextQuestion(categoryQuestionList, responseList, RelativeAfferentPupilDefectEventModel.QuestionCodes.ADPLaterality);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(RelativeAfferentPupilDefectEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    RelativeAfferentPupilDefectEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId, lateralityCode));

                    // Visit Data
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(result.VisitData, patientId, encounterId, lateralityCode));

                    // ADP
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.ADP, patientId, encounterId, lateralityCode));

                    if (result.ADP.IsAnswered)
                    {
                        // ADP Laterality
                        responsesToBeAdded.Add(MapTextQuestionToResponse(result.ADPLaterality, patientId, encounterId, lateralityCode));
                    }
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}