﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Static Threshold Perimetry Event View Manager
    /// </summary>
    public class StaticThresholdPerimetryEventViewManager : BaseEventViewManager<StaticThresholdPerimetryEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The static threshold perimetry view model</returns>
        public override StaticThresholdPerimetryEventModel GetDefaultViewModel()
        {
            var viewModel = new StaticThresholdPerimetryEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                StaticThresholdPerimetryEventModel.QuestionCodes.TestPerformed,
                StaticThresholdPerimetryEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                StaticThresholdPerimetryEventModel.QuestionCodes.TestResult,
                StaticThresholdPerimetryEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.EventComment);
            viewModel.ReferenceScale = MapToSingleSelectQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.ReferenceScale);
            viewModel.MeanDeviation = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.MeanDeviation);
            viewModel.PatternStandardDeviation = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.PatternStandardDeviation);
            viewModel.FixationLoses = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.FixationLoses);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The static threshold perimetry view model</returns>
        public override StaticThresholdPerimetryEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new StaticThresholdPerimetryEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.TestResult);
                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.EventComment);
                viewModel.ReferenceScale = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.ReferenceScale);
                viewModel.MeanDeviation = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.MeanDeviation);
                viewModel.PatternStandardDeviation = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.PatternStandardDeviation);
                viewModel.FixationLoses = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.FixationLoses);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(StaticThresholdPerimetryEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    StaticThresholdPerimetryEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Reference Scale
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.ReferenceScale, patientId,
                        encounterId, lateralityCode));

                    // Mean Deviation
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.MeanDeviation, patientId, encounterId,
                        lateralityCode));

                    // Pattern Standard Deviation
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.PatternStandardDeviation, patientId,
                        encounterId, lateralityCode));

                    // Fixation Loses
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.FixationLoses, patientId, encounterId,
                        lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}