﻿using System;
using System.Data;

/// <summary>
/// Summary description for ReferralDataManager
/// </summary>
namespace Managers.Referral
{
    /// <summary>
    /// The ReferralDataManager class handles data access for the Referral records in the database.
    /// </summary>
    public class ReferralDataManager
    {
        #region Member Variables

        private const string CONNECTION_NAME = "RegistryConnectionString";
        private const string GET_REFERRALS_WITH_CRITERIA = "MEVIR.usp_GetReferralsByCriteria";
        private const string GET_ENCOUNTERS_BY_REFERRALID = "MEVIR.usp_GetEncountersByReferralId";
        private const string GET_FACILITIES_BY_VISN_ID = "MEVIR.usp_Get_Facilities_by_Visn_ID";
        private const string GET_PATIENTSIDS_BY_REFERRAL_ID = "MEVIR.usp_Get_PatientSIDS_by_Referral_ID";
        private const string GET_REFERRAL_DATE_BY_ENCOUNTER_ID = "MEVIR.usp_Get_Referral_Date_By_Encounter_ID";
        private const string GET_CURRENT_USER_WORKLOAD_BY_ID = "MEVIR.usp_Get_Current_User_Workload_By_ID";

        private const string GET_USERS_BY_ROLE_CODE = "MEVIR.usp_Get_Registry_Users_By_Role_Code";

        #endregion

        /// <summary>
        /// Default Constructor
        /// </summary>
        public ReferralDataManager()
        {
        }

        #region Static Methods

        /// <summary>
        /// GetReferralsWithCriteria: calls the MEVIR.usp_GetReferralsByCriteria stored procedure with the passed parameters for a result set.
        /// </summary>
        /// <param name="statusId"></param>
        /// <param name="visnId"></param>
        /// <param name="facId"></param>
        /// <param name="patNameOrSSN"></param>
        /// <returns></returns>
        public static DataTable GetReferralsWithCriteria(int statusId, int? visnId, int? facId, string patNameOrSSN)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_REFERRALS_WITH_CRITERIA, new object[] { statusId, visnId, facId, patNameOrSSN });
        }

        /// <summary>
        /// Returns all encounters for the passed referral.
        /// </summary>
        /// <param name="referralId"></param>
        /// <returns></returns>
        public static DataTable GetEncountersByReferralId(int referralId, int statusId)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_ENCOUNTERS_BY_REFERRALID, new object[] { referralId, statusId });
        }

        /// <summary>
        /// Returns all facilities for the specified VISN
        /// </summary>
        /// <param name="visnId"></param>
        /// <returns></returns>
        public static DataTable GetFacilitiesByVisnId(int visnId)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_FACILITIES_BY_VISN_ID, new object[] { visnId });
        }

        /// <summary>
        /// GetPatientSIDsByReferralId: calls the 
        /// </summary>
        /// <param name="referralId"></param>
        /// <returns></returns>
        public static DataTable GetPatientSIDsByReferralId(int referralId)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_PATIENTSIDS_BY_REFERRAL_ID, new object[] { referralId });
        }

        /// <summary>
        /// GetReferralDateByEncounterID: returns the Referral date for the Encounter
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public static DateTime? GetReferralDateByEncounterID(int id)
        {
            try
            {
                object returnObj = SqlProvider.ExecuteScalar(CONNECTION_NAME, GET_REFERRAL_DATE_BY_ENCOUNTER_ID, new object[] { id });

                if (returnObj != null)
                {
                    if (returnObj is DateTime)
                    {
                        return (DateTime)returnObj;
                    }
                    else
                    {
                        return null;
                    }
                }
                else
                {
                    return null;
                }
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(ex.Message);
                return null;
            }
            //if (returnObj != null)
            //{
            //    DateTime dt;
            //    if (DateTime.TryParse(returnObj, out dt))
            //    {
            //        return dt;
            //    }
            //    else
            //    {
            //        return null;
            //    }
            //}
            //else
            //{
            //    return null;
            //}
        }

        /// <summary>
        /// Retrieves all encounters for the passed user - AssignedTo column.
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        public static DataTable GetCurrentUserWorkload(int userId, int statusId)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_CURRENT_USER_WORKLOAD_BY_ID, new object[] { userId, statusId });
        }

        /// <summary>
        /// Returns a list of users that belong to the passed role code.
        /// </summary>
        /// <param name="roleCode"></param>
        /// <returns></returns>
        public static DataTable GetUsersByRoleCode(string roleCode)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_USERS_BY_ROLE_CODE, new object[] { roleCode });
        }

        #endregion

        #region Cleanup -- unnecessary

        //public static List<ReferralView> GetReferralListWithCriteria(int statusId, int startRow, int maxRows)
        //{
        //    using (var ctx = new MEVIREntities())
        //    {
        //        bool useStatusFilter = false;

        //        if (statusId > 0)
        //        {
        //            useStatusFilter = true;
        //        }

        //        var temp = (
        //                    from em in ctx.ER_ENCOUNTER_MODS
        //                    join r in ctx.REFERRALs on em.REFERRAL_ID equals r.REFERRAL_ID
        //                    join s in ctx.sPatients on r.PATIENT_ID equals s.Patient_ID
        //                    join i in ctx.STD_INSTITUTION on em.InstitutionID equals i.ID
        //                    select new ReferralView()
        //                    {
        //                        REFERRAL_ID = r.REFERRAL_ID,
        //                        STD_REFERRALSTS_ID = r.STD_REFERRALSTS_ID,
        //                        REFERRAL_DATE = r.REFERRAL_DATE == null ? r.CREATED : (DateTime)r.REFERRAL_DATE,
        //                        PatientSSN = s.PatientSSN,
        //                        PatientName = s.PatientName,
        //                        VAMC_ID = i.ID,
        //                        STATIONNUMBER = i.STATIONNUMBER,
        //                        FACILITY = i.NAME
        //                    }
        //                   ).Distinct();


        //        if (useStatusFilter)
        //        {
        //            temp = temp.Where("EncounterStatusID = " + statusId);
        //        }

        //        temp = temp.OrderBy("PatientName");

        //        temp = temp.Skip(startRow).Take(maxRows);

        //        return temp.ToList();
        //    }
        //}

        #endregion
    }
}